@@ -1,3 +1,12 @@
+#  File tests/simulate_networkDynamic.R in package tergm, part of the Statnet suite
+#  of packages for network analysis, http://statnet.org .
+#
+#  This software is distributed under the GPL-3 license.  It is free,
+#  open source, and has the attribution requirements (GPL Section 7) at
+#  http://statnet.org/attribution
+#
+#  Copyright 2003-2014 Statnet Commons
+#######################################################################
 library(tergm)
 
 logit<-function(p)log(p/(1-p))
@@ -223,3 +232,92 @@ if(length((msm.sim%n%'net.obs.period')$observations)>1){
 }
 
 
+# --- check that adding and deleting vertices behaves properly ---
+g0<-network.initialize(20,dir=TRUE)
+g1<-san(g0~edges,target.stats=20,verbose=TRUE)
+# Simulate a networkDynamic from static 
+dynsim<-simulate(g1,formation=~edges,dissolution=~edges,
+                 coef.form=-9.326322,coef.diss=2.197225,
+                 time.slices=1,verbose=TRUE)
+# add more vertices
+add.vertices(dynsim,3)
+# simulate another step
+dynsim<-simulate(dynsim,formation=~edges,dissolution=~edges,
+                 coef.form=-9.326322,coef.diss=2.197225,
+                 time.slices=1,verbose=TRUE)
+# expect to see a pid defined, but will just be numeric values
+if (dynsim%n%'vertex.pid'!="tergm_pid"){
+  stop("simulate.networkDynamic net not add a vertex.pid to the network as expected")
+}
+if(!all(is.numeric(dynsim%v%"tergm_pid"))){
+  stop("simulate.networkDynamic did not create new numeric vertex pid as expected")
+}
+
+
+# add more (at this point there should be some non-numeric pids created)
+add.vertices(dynsim,3)
+if(all(is.numeric(dynsim%v%"tergm_pid"))){
+  stop("add.vertices did not create new non-numeric vertex pids as expected")
+}
+
+# simulate
+dynsim<-simulate(dynsim,formation=~edges,dissolution=~edges,
+                 coef.form=-9.326322,coef.diss=2.197225,
+                 time.slices=1,verbose=TRUE)
+# delete some vertices
+delete.vertices(dynsim,vid = 5:10)
+dynsim<-simulate(dynsim,formation=~edges,dissolution=~edges,
+                 coef.form=-9.326322,coef.diss=2.197225,
+                 time.slices=1,verbose=TRUE)
+
+# test with an externally defined PID  --------------
+
+# Simulate a networkDynamic from static 
+dynsim<-simulate(g1,formation=~edges,dissolution=~edges,
+                 coef.form=-9.326322,coef.diss=2.197225,
+                 time.slices=1,verbose=TRUE)
+dynsim%n%'vertex.pid'<-'letters'
+dynsim%v%'letters'<-LETTERS[1:20]
+# simulate another step
+dynsim<-simulate(dynsim,formation=~edges,dissolution=~edges,
+                 coef.form=-9.326322,coef.diss=2.197225,
+                 time.slices=1,verbose=TRUE)
+# expect to see a pid defined, but will just be numeric values
+if (dynsim%n%'vertex.pid'!="letters"){
+  stop("simulate.networkDynamic vertex pid missing")
+}
+if(!all(is.na(dynsim%v%"tergm_pid"))){
+  stop("simulate.networkDynamic created a tergm_pid when a vertex.pid already existed")
+}
+
+
+
+# ---- check summary.statistics.networkDynamic ---
+# this was giving error in #958
+my.nD <- network.initialize(100,directed=F)
+activate.vertices(my.nD, onset=0, terminus = 10)
+class(my.nD)
+summary(my.nD~isolates, at=1)
+
+
+# --- checking simulate with Inf coefficients ---
+# was giving error to #995
+n <- 50
+nw <- network.initialize(n, directed = FALSE)
+nw <- set.vertex.attribute(nw, "loc", rep(0:1, each = n/2))
+
+fit <- ergm(nw ~ edges + offset(nodemix("loc", base=c(1, 3))),
+            target.stats = 15,
+            offset.coef = -Inf)
+coef.form <- fit$coef
+coef.form[1] <- coef.form[1] - log(40-1)
+
+fit.sim <- simulate(fit)
+diag.sim <- simulate(fit.sim,
+                     formation = ~ edges + offset(nodemix("loc", base=c(1, 3))),
+                     dissolution = ~offset(edges),
+                     coef.form = coef.form,
+                     coef.diss = log(40-1),
+                     time.slices = 100,
+                     monitor = "formation",
+                     nsim = 1)
